
/* Copyright (c) 2007-2011, Stefan Eilemann <eile@equalizergraphics.com>.
 *                    2010, Cedric Stalder  <cedric.stalder@gmail.com>.
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License version 2.1 as published
 * by the Free Software Foundation.
 *  
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#include "objectSlaveDataOStream.h"

#include "log.h"
#include "masterCM.h"
#include "object.h"
#include "objectDataIStream.h"
#include "objectPackets.h"

namespace co
{
ObjectSlaveDataOStream::ObjectSlaveDataOStream( const ObjectCM* cm )
        : ObjectDataOStream( cm )
        , _commit( true )
{}

ObjectSlaveDataOStream::~ObjectSlaveDataOStream()
{}

void ObjectSlaveDataOStream::enableCommit( NodePtr node )
{
    _version = base::UUID( true );
    _setupConnection( node, false /* useMulticast */ );
    _enable();
}

void ObjectSlaveDataOStream::sendData( const void* buffer, const uint64_t size,
                                       const bool last )
{
    ObjectSlaveDeltaPacket packet;
    packet.commit = _commit;
    packet.instanceID = _cm->getObject()->getMasterInstanceID();
    ObjectDataOStream::sendData( packet, buffer, size, last );

    if( last )
        _commit = base::UUID( true /* generate */ );
}

}
